# Documentation string. I wonder what ever refers to it? 

__doc__ = """ZFDocument product module.

This module provides the methods and classes that Zope needs for the product. 
__init__.py is responsible for registering these methods and classes.

See README.txt for more details."""

# Version string. Usually updated automatically by CVS. 

__version__ = '0.1' # I used to wonder whether Zope checked these. Nope. 

# Now, you can compress these imports down to three lines, but that makes it 
# harder to document it as you go. Here's the long-winded version: 

from Globals import HTML, HTMLFile
from Globals import MessageDialog
from Globals import Persistent

import OFS.SimpleItem
import Acquisition
import AccessControl.Role

from OFS.DTMLDocument import DTMLDocument
from OFS.DTMLMethod import DTMLMethod, decapitate
from urllib import quote, unquote

import re
import string
from xmlrpclib import True, False, Server
from PikeCompatibility import PikeEdit

"""Step #1: Provide "constructors" for Zope's management interface. 
__init__ is all very well and good for Python, but doesn't provide 
a nice HTML interface or accept arguments from an HTTP request, so 
you need to provide some functions to act as a middle-man between 
Zope and your class.

Use HTMLFile to produce a [pseudo-]function which hands back an 
appropriate DTML document based on a file sitting in your product 
directory. 
"""

"""Step #2: Provide the class for your ZFDocument object."""

class ZFDocument(DTMLDocument, PikeEdit): 
    """
    ZFDocument object. 
    """
    meta_type = 'ZF Document' # what do people think they're adding?
    manage_main = HTMLFile('zfdocEdit', globals())
    manage_workspace = manage_editForm = manage_main

default_zfdocument = """<dtml-var standard_html_header>
<h2><dtml-var title_or_id></h2>
<p>
This is the <dtml-var document_id> Document 
</p>
<dtml-if "may_edit(REQUEST)">
<dtml-var "edit_in_browser_button(REQUEST)">
<dtml-var "edit_in_pike_button(REQUEST)">
</dtml-if>
<dtml-var standard_html_footer>"""

def manage_addZFDocument(self, id, title='', file='', REQUEST=None, submit=None):
    """
    Add a ZFDocument object with the contents of file. If
    'file' is empty, default text is used.
    """
    if type(file) is not type(''): file=file.read()
    if not file:
        file = default_zfdocument
    ob=ZFDocument(file, __name__=id)
    ob.title=title
    id=self._setObject(id, ob)

    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_main')
    return ''

manage_addZFDocumentForm=HTMLFile('zfdocAdd', globals())

class ZFMethod(DTMLMethod, PikeEdit): 
    """
    ZFMethod object. 
    """
    meta_type = 'ZF Method' # what do people think they're adding?
    manage_main = HTMLFile('zfdocEdit', globals())
    manage_workspace = manage_editForm = manage_main

default_zfmethod = """<dtml-var standard_html_header>
<h2><dtml-var title_or_id></h2>
<p>
This is the <dtml-var document_id> Document 
</p>
<dtml-var standard_html_footer>"""

def manage_addZFMethod(self, id, title='', file='', REQUEST=None, submit=None):
    """
    Add a ZFMethod object with the contents of file. If
    'file' is empty, default text is used.
    """
    if type(file) is not type(''): file=file.read()
    if not file:
        file = default_zfmethod
    ob=ZFMethod(file, __name__=id)
    ob.title=title
    id=self._setObject(id, ob)

    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_main')
    return ''

manage_addZFMethodForm=HTMLFile('zfmethodAdd', globals())

class ZFTemplateDocument(ZFDocument):
    """
    ZFTemplate
    """
    meta_type = 'ZF Template Document'
    manage_main = HTMLFile('zftemplateEdit', globals())
    manage_workspace = manage_editForm = manage_main
    isZFTemplate = 1

    def getTemplate(self):
        """Find a template in the current folder and above"""
        if hasattr(self, 'useTemplate'):
            useTemplateName = self.useTemplateName
        else:
            useTemplateName = None
            
        while self is not None:
            if hasattr(self, 'objectValues'):
                for o in self.objectValues():
                    if hasattr(o, 'template') and o != self:
                        if useTemplateName is not None:
                            if hasattr(o, 'id'):
                                if type(o.id) != type(''):
                                    thisTemplateName = o.id()
                                else:
                                    thisTemplateName = o.id
                            else:
                                thisTemplateName = o.template
                            if useTemplateName == thisTemplateName:
                                return o
                        else:
                            return o
            if hasattr(self, 'aq_parent'):
                self = self.aq_parent
            else:
                self = None
        if useTemplateName is not None:
            raise AttributeError("No Template found: " + useTemplateName)
        else:
            raise AttributeError("No Template found!")

    def __call__(self, client=None, REQUEST={}, RESPONSE=None, **kw):
        """
        If we can find a template (that is not also us), render
        ourselves within the template.

        Note that templates can themselves be ZFTemplateDocument's,
        but they cannot specify themselves as their template.
        Gotta break the loop somewhere.
        """
        kw['template_id']    = self.id
        kw['template_title'] = self.title
        kw['template_self'] = self
        template = self.getTemplate()
        kw['body'] = apply(DTMLDocument.__call__, (self, client, REQUEST, RESPONSE), kw)
        if hasattr(template, 'isZFTemplate'):
            return apply(ZFTemplateDocument.__call__, (template, client, REQUEST, RESPONSE), kw)
        else:
            return apply(DTMLDocument.__call__, (template, client, REQUEST, RESPONSE), kw)

default_zftemplate = """<h2><dtml-var title_or_id></h2>
<p>
This is the <dtml-var document_id> Document 
</p>
<dtml-if "may_edit(REQUEST)">
<dtml-var "edit_in_browser_button(REQUEST)">
<dtml-var "edit_in_pike_button(REQUEST)">
</dtml-if>"""

def manage_addZFTemplateDocument(self, id, title='', file='', REQUEST=None, submit=None):
    """
    Add a ZFTemplateDocument object with the contents of file. If
    'file' is empty, default text is used.
    """
    if type(file) is not type(''):
        file = file.read()
    if not file:
        file = default_zftemplate
    ob = ZFTemplateDocument(file, __name__=id)
    ob.title = title
    id = self._setObject(id, ob)

    if REQUEST is not None:
        try:
            u = self.DestinationURL()
        except:
            u = REQUEST['URL1']
        if submit == " Add and Edit ":
            u = "%s/%s" % (u,quote(id))
        REQUEST.RESPONSE.redirect(u + '/manage_main')
    return ''

manage_addZFTemplateDocumentForm=HTMLFile('zftemplateAdd', globals())

